﻿package Plugins::Certify;

use Net::LDAP::LDIF;
use Net::LDAP::Entry;
use Config::General;
use MIME::Base64;
use XML::Simple;

$VERSION  = 1.0;
my $plugin_name = 'Certify';
$this_pkg = (caller(0))[0];
my $redni_broj_certificiranja = 2;
my $realm = base2realm( $config::base_dn);

my $conf = new Config::General(
    -ConfigFile      => $config::plugins_dir . '/'.$plugin_name.'.conf',
    -InterPolateVars => TRUE,
    -AutoLaunder     => TRUE
);
my %cfg = $conf->getall;

$config::cert_file_prefix = $config::log_dir . '/' . $cfg{'cert_file_prefix'};
$config::cert_file_ext    = $cfg{'cert_file_ext'};
$config::cert_script_path      = $config::plugins_dir.'/'.$cfg{'script_name'};
$config::cert_plain_text  = $cfg{'dump_plain_text'};
@valid_ips                = keys %{ $cfg{'valid_ip'} };


AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': getting IP list: ' . join(", ", @valid_ips)) if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': script_path=' . $config::cert_script_path)  if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': dump_plain_text=' . $config::cert_plain_text)  if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . '::'.$plugin_name.': cert_file=' . $config::cert_file_prefix . '_' . $redni_broj_certificiranja . '_' . $realm . $config::cert_file_ext)
  if $config::aosi_debug_level > 1;

##################################################################
#  Check if IP is in the list
##################################################################
sub cert_is_valid {
    my ($in) = @_;
    my ($ip, $ok);

    $ok = 0;
    foreach $ip (@valid_ips) {
        chomp($ip);
        if ($ip eq $in) {
            $ok++;
        }
    }
    AOSICFG::write_ws_log('| Sync IP ' . $in . ' is valid: ' . $ok) if $config::aosi_debug_level > 2;
    return $ok;
}

##################################################################
#  Extraxt realm from base DN
##################################################################
sub base2realm {
    my ($base) = @_;
    my ($temp, $realm);

    ($realm) = split(/,/, $base);
    ($temp, $realm) = split(/=/, $realm);

    return $realm;
}

sub AOSI::ldapCertData {
    my ($service, $user, $pwd, $base, $type) = @_;
    my ($log, $error, $code, $out_msg, $cert, $cert_file_b64,  $cert_file_txt, @cert, $realm, $dn, $sys_b64, $sys);
    my ($dummy, $dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks);
    $log = "ldapCertData";
    $code = 0;
    $error = '';

    AOSICFG::write_ws_log("/ $log(in):user:$user;base:$base;type:$type;") if $config::aosi_debug_level > 0;
    AOSICFG::write_ws_log("| $log: IP:$AOSI::client_ip;")                 if $config::aosi_debug_level > 1;

    $realm = base2realm($base);
    $cert_file_b64 = $config::cert_file_prefix . "_" . $redni_broj_certificiranja . "_" . $realm . '_base64' . $config::cert_file_ext;
    $cert_file_txt = $config::cert_file_prefix . "_" . $redni_broj_certificiranja . "_" . $realm . $config::cert_file_ext;
    if ($user =~ /=/g) {
        $dn = $user . ',' . $base;
    }
    else {
        $dn = 'uid=' . $user . ',' . $base;
    }
    AOSICFG::write_ws_log("| $log: dn=$dn;") if $config::aosi_debug_level > 1;

    if (cert_is_valid($AOSI::client_ip)) {
        # Save Base64 data
        # Perltidy escape
        #<<<
        $run = $config::cert_script_path . ' -b -n -d -o '.$cert_file_b64.' 2>&1';
        #>>>
        $sys_b64 = qx($run);
        AOSICFG::write_ws_log("| $log: run1=$run;sys=$sys_b64;") if $config::aosi_debug_level > 2;

        if($config::cert_plain_text == 1){
            # Save text data
            # Perltidy escape
            #<<<
            $run = $config::cert_script_path . ' -n -d -o '.$cert_file_txt.' 2>&1';
            #>>>
            $sys = qx($run);
            AOSICFG::write_ws_log("| $log: run2=$run;sys=$sys;") if $config::aosi_debug_level > 2;
        }


        if ($sys_b64 =~ /.*OK!/g) {
            open(CERT, $cert_file_b64)
              or do { $code = -20; $out_msg = '<status code="' . $code . '">Can not open file ' . $cert_file_b64 . '</status>'; $error = $!; };
            if ($code == 0) {
                @cert = <CERT>;
                close(CERT);
                $cert = join("", @cert);
                ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($cert_file_b64);
                $cert = XML::Simple::escape_value($dummy, $cert);
                $out_msg = '<attribute ldapname="cert_data"><value>' . $cert . '</value></attribute>';
                $out_msg .= '<attribute ldapname="size"><value>' . $size . '</value></attribute>';
                $out_msg .= '<attribute ldapname="time"><value>' . AOSI::t2t($mtime) . '</value></attribute>';
                $out_msg .= '<attribute ldapname="pm_ver"><value>' . $VERSION . '</value></attribute>';
                $out_msg .= '<attribute ldapname="cert_ver"><value>' . $redni_broj_certificiranja . '</value></attribute>';
            }
            close(CERT);
        }
        else {
            $code    = -20;
            $out_msg = '<status code="' . $code . '">Script ('.$config::cert_script_path.') execution error (' . $sys . ')!</status>';
        }
    }
    else {
        $code    = -23;
        $error   = 'You are not allowed to call this function!';
        $out_msg = '<status code="' . $code . '">' . $error . '</status>';
    }

    $hash->{'code'}  = $code;
    $hash->{'error'} = $error;
    $hash->{'result'} =
        '<?xml version="1.0"?><ldap action="' . $log . '" version="' . $config::aosi_version . '"><entry dn="' . $dn . '">' . $out_msg . '</entry></ldap>';


    AOSICFG::write_ws_log("\\ $log(out):" . AOSICFG::dump_hash($hash)) if $config::aosi_debug_level > 0;
    return AOSI::hash2data($hash, { 'code' => 'int' });
}

1;
