#!/usr/bin/perl
BEGIN {
	unshift(@INC,'/etc/aosi');
}
use config;
#use warnings;
use utf8;
use MIME::Base64;
use Net::LDAP;
use Time::Local;
use MIME::Lite;
use Getopt::Long;

$print_detail=0;
$base64_only=0;
$sendmail=0;
$opt_h = 0;
$realm = '';

$res = GetOptions(
"detail|d" => \$print_detail,
"base64|b" => \$base64_only,
"sendmail|s" => \$sendmail,
"outfile|o=s" => \$out_file,
"realm|r=s" => \$realm,
"help|h" => \$opt_h,
);


if($opt_h){
	print "usage: $0 [--detail|-d] [--base64|-b] [--sendmail|-s] [--outfile <filename>|-o <filename>] [--realm <realm>|-r <realm>]\n";
	print "    --detail     detaljni opis nepravilnosti\n";
	print "    --base64     ispisuje se samo Base64 dio\n";
	print "    --sendmail   salje se mail AAI timu\n";
	print "    --outfile    rezultat se sprema u datoteku <filename> umjesto ispisa na zaslon\n";
	print "    --realm      provjerava se LDAP za navedeni realm (umjesto \$config::base_dn u /etc/aosi/config.pm)\n";
	exit(1);
}

$email_adresa='aosi@aaiedu.hr';
$verzija=0.99;

if($realm eq ''){
	$base_dn=$config::base_dn;
}
else {
	if($realm =~ /(.*)\.hr$/){
		$realm = $1;
	}
	$base_dn = 'dc='.$realm.',dc=hr';
}
$ldap_user=$config::ldap_user;
($ldap_ip,$ldap_port)=split(/:/,$config::ldap_server);

#procitati ldap administracijski password
$pwd=_get_pwd($config::ldap_pwd_file);

$br_neisp_mail=0;
$br_dupli_mail=0;
$br_kor_krivi_email=0;
$br_oib=0;
$br_gost_bez_oiba=0;
$br_gost_neisp_oib=0;
$br_jmbg=0;
$br_neisp_oib=0;
$br_neisp_jmbg=0;
$br_dupli_unique_number=0;
$br_isteklih=0;
$br_studenata_ne_isticu=0;
$br_kor_dupli_unique_number=0;
$br_neuskladjen_oib=0;
$br_nije_gost_nema_oib=0;

unless ($ldap = Net::LDAP->new($ldap_ip, port => $ldap_port, version => 'LDAPv3')){
	die("Greska! Ne mogu kontaktirati LDAP posluzitelj.\n");
}
unless ($mesg = $ldap->bind($ldap_user, version=>3, password => $pwd)){
	die("Greska! Neispravna zaporka. Ne mogu se spojiti na LDAP posluzitelj.\n");
}
$mesg->code && die("Greska 1:" . $mesg->error . " (" . $mesg->code . ")\n");

#$ldap_filter='(uid=*)';
$ldap_filter='(&(uid=*)(objectclass=hrEduPerson))';
unless($mesg = $ldap->search (base => $base_dn, sizelimit => 0, scope  =>'sub', attrs => ['uid','mail','hrEduPersonUniqueNumber','hrEduPersonExpireDate','hrEduPersonPrimaryAffiliation','hrEduPersonOIB'], filter => $ldap_filter, timelimit => 360, async => 1)){
	die("Greska 2:" . $mesg->error."\n");
}
$mesg->code && die("Greska 3:" . $mesg->error. " (" . $mesg->code . ")\n");

$broj_kor=0;

if($base64_only == 0){
    $result="\n-------------------\nRezultati testa za base dn: $base_dn.\nVrijeme: " . vrijeme() . "\nVerzija:$verzija\n-------------------\n\n";

    if($print_detail) {$result .= "---detaljne informacije---\n"}
}
else {
    $result = "";
}

#dan prije 90 dana
$granica=ldapdate(time - (90*24*3600));

while (my $ent = $mesg->shift_entry) {
	$broj_kor++;
	my $uid=$ent->get_value('uid');
	my $oib=$ent->get_value('hrEduPersonOIB');
	my $exp_date=$ent->get_value('hrEduPersonExpireDate');
	my $aff=$ent->get_value('hrEduPersonPrimaryAffiliation');
	my $mail_ref=$ent->get_value('mail', asref => 1);
	my $un_ref=$ent->get_value('hrEduPersonUniqueNumber', asref => 1);
	my $krivi_email=0;
	my $dupli_unique_number=0;
	#provjera svih vrijednosti e-mail adrese
	foreach my $m (@$mail_ref) {
		#je li ispravna
		unless(valid_email($m)==1){
			$br_neisp_mail++;
			$krivi_email++;
			if($print_detail) {$result .= "neispravan e-mail: uid:$uid, e-mail:$m\n"}
		}
		#je li duplicirana
		if($mailovi{$m}==1){
			$mailovi{$m}++;
			$br_dupli_mail+=2;
			$krivi_email++;
			if($print_detail) {$result .= "dupliciran e-mail: uid:$uid, e-mail:$m\n"}
		} elsif ($mailovi{$m}>1){
			$mailovi{$m}++;
			$br_dupli_mail++;
			$krivi_email++;
		if($print_detail) {$result .= "visestruk e-mail: uid:$uid, e-mail:$m\n"}
		} else {
			$mailovi{$m}=1;
		}
	}
	
	#ako korisnik ima bar jednu vrijednost mail atributa krivu, brojimo ga
	#broje se korisnici s krivim e-mailovima, a ne sami e-mailovi
	if($krivi_email>0){
		$br_kor_krivi_email++;
	}
	
	#je li datum istekao prije vise od tri mjeseca (90 dana)
	if($exp_date eq 'NONE'){
		#broji se ako ucenik ili student ima postavljen NONE za datum isteka
		if($aff =~ /[sS]tudent|[uU].+enik/){
			$br_studenata_ne_isticu++;
			if($print_detail) {$result .= "ucenik ili student e-identitet ne istice: uid:$uid, hrEduPersonPrimaryAffiliation:$aff, hrEduPersonExpireDate:$exp_date\n"}
		}
	} else {
		if($exp_date lt $granica){
			$br_isteklih++;
			if($print_detail) {$result .= "e-identitet istekao prije vise od 90 dana: uid:$uid, hrEduPersonExpireDate:$exp_date\n"}
		}
	}
	
	#provjera valjanosti OIB-a
	if($oib ne 'NONE'){
		#ako je oib provjeriti je li ispravan
		if(valid_oib($oib)){
			my $oib_found;
			foreach my $u (@$un_ref) {
				if($u =~ /^OIB:/){
					$oib_found=1;
					if($oib eq substr($u,-11)){
						$br_oib++;
					} else {
						$br_neisp_oib++;
						if($print_detail) {$result .= "oib upisan u atributu hrEduPersonOIB i u atributu hrEduPersonUniqueNumber nisu isti: uid:$uid, $u!=$oib\n"}
					}
				}
			}
			if($oib_found!=1){
				$br_neuskladjen_oib++;
				if($print_detail) {$result .= "oib je upisan u hrEduPersonOIB ali ne i u hrEduPersonUniqueNumber: uid:$uid, $oib\n"}
			}
		} else {
			$br_neisp_oib++;
			if($print_detail) {$result .= "neispravan oib: uid:$uid, $oib"}
			if($aff =~ /[gG]ost/){
				#gosti koji imaju neispravan OIB se posebno broje
				$br_gost_neisp_oib++;
				if($print_detail) {$result .= " ** GOST"}
			}
			if($print_detail) {$result .= "\n"}
		}
	} else {
		if($aff =~ /[gG]ost/){
			#gosti koji nemaju upisan OIB se posebno broje
			$br_gost_bez_oiba++;
		} else {
			$br_nije_gost_nema_oib++;
			if($print_detail) {$result .= "korisnik nije gost, a nema upisan hrEduPersonOIB: uid:$uid, hrEduPersonPrimaryAffiliation: $aff\n"}
		}
	}
	#provjera svih vrijednosti hrEduPersonUniqueNumber
	foreach my $u (@$un_ref) {
		
		#ako je jmbg provjeriti je li ispravan
		if ($u =~ /^JMBG:/){
			if(valid_jmbg(substr($u,-13))){
				$br_jmbg++;
			} else {
				$br_neisp_jmbg++;
				if($print_detail) {$result .= "neispravan JMBG: uid:$uid, $u\n"}
			}
		}
		#je li dupliciran
		if($unique_number{$u}==1){
			$unique_number{$u}++;
			$br_dupli_unique_number+=2;
			$dupli_unique_number++;
			if($print_detail) {$result .= "dupliciran hrEduPersonUniqueNumber: uid:$uid, $u\n"}
		} elsif ($unique_number{$u}>1){
			$unique_number{$u}++;
			$br_dupli_unique_number++;
			$dupli_unique_number++;
			if($print_detail) {$result .= "visestruk hrEduPersonUniqueNumber: uid:$uid, $u\n"}
		} else {
			$unique_number{$u}=1;
		}
	}
	
	#ako korisnik ima bar jednu vrijednost hrEduPersonUniqueNumber atributa krivu, brojimo ga
	if($dupli_unique_number>0){
		$br_kor_dupli_unique_number++;
	}
	
}
if($print_detail) {$result .= "---detaljne informacije---\n\n"}
$org_data='';
$ldap_filter='(objectclass=hrEduOrg)';
unless($mesg = $ldap->search (base => $base_dn, sizelimit => 0, scope  =>'sub', filter => $ldap_filter, timelimit => 360)){
	die("Greska 4:" . $mesg->error."\n");
}
$mesg->code && die die("Greska 5:" . $mesg->error."\n");

while (my $ent = $mesg->shift_entry) {
	my @attr = $ent->attributes;
	foreach my $a ( @attr ) {
		my $a_ref=$ent->get_value($a, asref => 1);
		$org_data .= $a . ': ' . join(":#:",@$a_ref) . "\n";
	}
}

$posto_isteklih=sprintf("%.2f", (($br_isteklih+$br_studenata_ne_isticu)*100/$broj_kor));
$posto_kor_neispravan_email=sprintf("%.2f", ($br_kor_krivi_email*100/$broj_kor));
$posto_kor_ima_oib=sprintf("%.2f", ($br_oib*100/$broj_kor));
$posto_kor_dupli_unique_number=sprintf("%.2f", ($br_kor_dupli_unique_number*100/$broj_kor));

$result.="Broj korisnika u imeniku:$broj_kor\nBroj neispravnih e-mail adresa:$br_neisp_mail\nBroj visestrukih e-mail adresa:$br_dupli_mail\nBroj korisnika s neispravom e-mail adresom:$br_kor_krivi_email\nBroj korisnika s upisanim ispravnim OIB-om:$br_oib\nBroj korisnika s upisanim neispravnim OIB-om:$br_neisp_oib\nBroj korisnika koji nisu gosti, a nemaju upisan OIB:$br_nije_gost_nema_oib\nBroj korisnika s neuskladjenim OIB-om u atributima hrEduPersonOIB i hrEduPersonUniqueNumber:$br_neuskladjen_oib\nBroj gostiju bez OIB-a:$br_gost_bez_oiba\nBroj korisnika s upisanim ispravnim JMBG-om:$br_jmbg\nBroj gostiju s neispravnim OIB-om:$br_gost_neisp_oib\nBroj korisnika s upisanim neispravnim JMBG-om:$br_neisp_jmbg\nBroj ponovljenih vrijednosti hrEduPersonUniqueNumber-a:$br_dupli_unique_number\nBr e-identiteta isteklih prije vise od 90 dana:$br_isteklih\nBroj studenata ili ucenika ciji e-identiteti ne isticu:$br_studenata_ne_isticu\n";
$result.="\nPostotak isteklih e-identiteta (ubrojani i studentski i ucenicki bez podatka o isteku):$posto_isteklih\nPostotak korisnika s neispravnom ili dupliciranom e-mail adresom:$posto_kor_neispravan_email\nPostotak korisnika koji imaju upisan (ispravan) OIB:$posto_kor_ima_oib\nPostotak korisnika koji imaju upisan nejedinstven hrEduPersonUniqueNumber:$posto_kor_dupli_unique_number\n";
$result.="\nPodaci o ustanovi:\n$org_data";

if($base64_only == 0){
    $result.= "\n----base64----\n" . encode_base64($result) . "----base64----\n";
}
else {
    $result = encode_base64($result)."\n";
}

if($out_file ne ''){
    open(OUT, ">$out_file") or die "$0: cannot open $out_file for writing ($!)\n";
    print OUT $result;
    close(OUT);
    if($base64_only){
        print "All OK!\n";
    }
}
else {
    print $result;
}

if($sendmail == 1){
    print "Zelite li gore ispisane rezultate testa poslati Koordinatoru AAI\@EduHr sustava na adresu:$email_adresa (Da/Ne) [Ne]? ";
    chomp($ans = <STDIN>);
    if ($ans =~ /^[dD][aA]$/) {
        my $message = MIME::Lite->new(
        To      => $email_adresa,
        Subject => '[CERT2] ' . $base_dn,
        Data    => $result
      );
      $message->attr("content-type"         => "text/plain");
        $message->attr("content-type.charset" => "utf-8");
        $message->send || die "Slanje maila nije uspjelo!";
      print "Rezultati testa su poslani e-mailom na adresu $email_adresa.\n";
    } else {
      print "Rezultati testa nisu poslani.\n";
    }
}

sub valid_oib {
	return 0 unless $_[0] =~ /^\d{11}$/;
	my @chars = split(//, $_[0]);
	my $P = 10;
	my $S;
	for my $a (@chars) {
		$S = $P%11 + $a;
		$P = ($S%10 || 10) * 2;
	}
	return $S%10 == 1;
}
sub valid_jmbg {
    my($jmbg) = @_;
    my($out, $len, $last1, $rest, $last2);

    $last1 = substr($jmbg,-1);
    $len = length($jmbg);
    $rest = substr($jmbg, 0, $len-1);
    $last2 = jmbg_ctrl_num($rest);
    $out = $last1 eq $last2 ? 1 : 0;

    return $out;
}

sub jmbg_ctrl_num {
    my($in_stg) = @_;
    my($num, $len, $sum, $i, $rest, $out);

    $num = $in_stg;
    $num =~ s/\D//g;

    $len = length($num);

    $sum = 0;
    for ($i = 0; $i < $len; $i++) {
        $cur = substr($num, $i, 1);
        $tmp1 = $i % 6;
        $tmp2 = 7 - $tmp1;
        $tmp3 = $tmp2 * $cur;
        $sum += $tmp3;
    }
    $rest = $sum % 11;
    $out = $rest == 0 ? 0 : 11 - $rest;

    return $out;
}

sub valid_email {
	my ($mail) = @_;
	if ($mail =~ /^[\w\-\_\.\+!#\$%&'\*\+\/=\?\^_`]+\@([\w\-\_]+\.)+[a-zA-Z]{2,}$/){
		return 1;
	} else {
		return 0;
	}
}

sub _get_pwd {
    my ($file) = @_;
    my ($real);

    open(FILE, "$file") or die "$0: can not open file $file for reading ($!)\n";
    $real = <FILE>;
    close(FILE);
    chomp($real);
    return $real;
}
sub ldapdate {
		my ($timestamp)=@_;
        my ($second, $minute, $hour, $dayOfMonth, $month, $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings) = localtime($timestamp);
        my $year = 1900 + $yearOffset;
        $month++;
        $month= substr "0$month", -2;
        $dayOfMonth= substr "0$dayOfMonth", -2;
        return "$year$month$dayOfMonth";
}

sub vrijeme {
        my ($second, $minute, $hour, $dayOfMonth, $month, $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings) = localtime();
        my $year = 1900 + $yearOffset;
        $month++;
        $month= substr "0$month", -2;
        $dayOfMonth= substr "0$dayOfMonth", -2;
        return "$dayOfMonth. $month. $year. $hour:$minute:$second";
}
